import { queryOffsetLimitParser } from '../../common/parsers/queryParams';
import { z } from 'zod';
import { userIdParser } from '../../user/parsers';
import { currencyAmountParser } from '../../currencies/parsers';
import type { UserBooksReportResponseSerialized, UsersBooksReportResponseSerialized } from './types';
import { booleanQueryParser } from '../../parsers';

export const usersBooksQueryParser = queryOffsetLimitParser.merge(
  z.object({
    userId: userIdParser.optional(),
    minOutstandingPayout: currencyAmountParser.optional(),
    hasPayoutMethod: booleanQueryParser,
  })
);

export const userBooksQueryParser = queryOffsetLimitParser.merge(
  z.object({
    // TODO filter per user
  })
);

export const userBooksParamsParser = queryOffsetLimitParser.merge(
  z.object({
    userId: userIdParser,
  })
);

export const usersBooksResultParser = z.any().transform((o) => o as UsersBooksReportResponseSerialized);
export const userBooksResultParser = z.any().transform((o) => o as UserBooksReportResponseSerialized);

export const userBooksUsersResultParser = z.array(userIdParser);
export const userBooksUsersQueryParser = z.object({
  startsWith: z.string().optional().default(''),
});
