import {
  BOOK_QUEUED_AUTHOR,
  BOOK_DEPRIVATIONS_AUTHOR,
  BOOK_OPS,
  BOOK_PAYINS_AUTHOR,
  BOOK_PAYINS_ENJOYER,
  BOOK_PAYOUTS_AUTHOR,
  BOOK_PAYOUTS_ENJOYER,
  BOOK_PENDING_AUTHOR,
  BOOK_PENDING_ENJOYER,
  BOOK_REFUNDS_AUTHOR,
  BOOK_REFUNDS_ENJOYER,
  BOOK_TYPES,
  PLACE_KINDS,
  PLACE_OPERATION,
} from './constants';
import { Subset } from '../utils/types';

export type BookType = typeof BOOK_TYPES[number];
// from the Company perspective
export const bookTypeToPlaceKind = {
  // obviously an asset
  [BOOK_PAYINS_AUTHOR]: 'asset',
  [BOOK_PAYINS_ENJOYER]: 'asset',
  // the refund is done against company assets
  [BOOK_REFUNDS_AUTHOR]: 'asset',
  [BOOK_REFUNDS_ENJOYER]: 'asset',
  // the payout is done from company assets
  [BOOK_PAYOUTS_AUTHOR]: 'asset',
  [BOOK_PAYOUTS_ENJOYER]: 'asset',
  // we're still about to pay it
  [BOOK_QUEUED_AUTHOR]: 'liability',
  // what was taken from them
  [BOOK_DEPRIVATIONS_AUTHOR]: 'asset',
  // we are liable to pay those out at some point
  [BOOK_PENDING_AUTHOR]: 'liability',
  [BOOK_PENDING_ENJOYER]: 'liability',
} as const;
// eslint-disable-next-line @typescript-eslint/no-unused-vars
type AssertBookTypeToPlaceKind1 = Subset<
  typeof bookTypeToPlaceKind,
  {
    [key in BookType]: PlaceKind;
  }
>;
// eslint-disable-next-line @typescript-eslint/no-unused-vars
type AssertBookTypeToPlaceKind2 = Subset<BookType, keyof typeof bookTypeToPlaceKind>;
export type BookTypeToPlaceKind = typeof bookTypeToPlaceKind;
export type PlaceKind = typeof PLACE_KINDS[number];

export type PlaceOperation = typeof PLACE_OPERATION[number];

export type BookOp = typeof BOOK_OPS[number];
export const placeKindAndBookOpToPlaceOperation: {
  [kind in PlaceKind]: {
    [op in BookOp]: PlaceOperation;
  };
} = {
  asset: {
    debit: 'increase',
    credit: 'decrease',
  },
  liability: {
    debit: 'decrease',
    credit: 'increase',
  },
};

export type PlaceKindAndBookOpToPlaceOperation = typeof placeKindAndBookOpToPlaceOperation;

enum JournalEntryIdBrand {
  _ = '',
}

export type JournalEntryId = string & JournalEntryIdBrand;

enum BookEntryIdBrand {
  _ = '',
}

export type BookEntryId = string & BookEntryIdBrand;

enum ShelfIdBrand {
  _ = '',
}

export type ShelfId = string & ShelfIdBrand;

enum BookIdBrand {
  _ = '',
}

export type BookId = string & BookIdBrand;
