import { CollectiveId } from '../collective/types';
import { BlogId } from '../blog/id.types';
import { CHANNEL_TYPES } from './constants';
import { MixspaceId } from '../mixspace/id.types';
import { Subset } from '../utils/types';
import { z } from 'zod';
import { channelCompositeIdParser, channelIdParser } from './parsers';

export type ChannelId = CollectiveId | BlogId | MixspaceId;
// we do not move payments thru mixspaces YET
export type ChannelIdClassicPayable = Exclude<ChannelId, MixspaceId>;
export type ChannelType = typeof CHANNEL_TYPES[number];

type ChannelIdOfTypeMapping = {
  collective: CollectiveId;
  blog: BlogId;
  mixspace: MixspaceId;
}; // TODO 'satisties' of 4.9

// eslint-disable-next-line @typescript-eslint/no-unused-vars
type AssertChannelIdOfTypeMapping1 = Subset<keyof ChannelIdOfTypeMapping, ChannelType>;
// eslint-disable-next-line @typescript-eslint/no-unused-vars
type AssertChannelIdOfTypeMapping2 = Subset<ChannelType, keyof ChannelIdOfTypeMapping>;

export type ChannelIdOfType<T extends ChannelType> = ChannelIdOfTypeMapping[T];

// eslint-disable-next-line @typescript-eslint/no-unused-vars
type AssertChannelId1 = Subset<z.infer<typeof channelIdParser>, ChannelId>;
// eslint-disable-next-line @typescript-eslint/no-unused-vars
type AssertChannelId2 = Subset<ChannelId, z.infer<typeof channelIdParser>>;

export type ChannelCompositeId = z.infer<typeof channelCompositeIdParser>;

// eslint-disable-next-line @typescript-eslint/no-unused-vars
type AssertChannelCompositeIdType1 = Subset<ChannelCompositeId['type'], ChannelType>;
// eslint-disable-next-line @typescript-eslint/no-unused-vars
type AssertChannelCompositeIdType2 = Subset<ChannelType, ChannelCompositeId['type']>;

// eslint-disable-next-line @typescript-eslint/no-unused-vars
type AssertChannelCompositeIdId1 = Subset<ChannelCompositeId['id'], ChannelId>;
// eslint-disable-next-line @typescript-eslint/no-unused-vars
type AssertChannelCompositeIdId2 = Subset<ChannelId, ChannelCompositeId['id']>;
