import { z } from 'zod';
import { MAX_HISTORICAL_DATE_MS } from '../../constants';
import { currencyAmountParser, currencyParser } from '../../currencies/parsers';
import { channelIdParser } from '../../channel/parsers';
import { userIdParser } from '../../user/parsers';
import type { PayinRouteResult } from './types';
import { makePositiveIntegerQueryParser } from '../../common/parsers/queryParams';
import { routingStrategyQueryParser } from '../../graph/strategy/parsers';

export const routePacketQueryParser = z
  .object({
    userId: userIdParser,
    payeeId: channelIdParser,
    amt: currencyAmountParser,
    unit: currencyParser,
    when: makePositiveIntegerQueryParser('When')
      .refine((n) => n <= MAX_HISTORICAL_DATE_MS, {
        message: `"When" must be less than ${MAX_HISTORICAL_DATE_MS}`,
      })
      .optional(),
  })
  .merge(routingStrategyQueryParser);

export const routePacketResultParser = z.any(/* stub */).transform((v) => v as PayinRouteResult);
