import { z } from 'zod';
import {
  payoutMethodDeleteParamsParser,
  payoutMethodListQueryParser,
  payoutMethodListResultParser,
  payoutMethodStripeAuthorizeResultParser,
  payoutMethodStripeTokenBodyParser,
  payoutMethodUpsertParamsParser,
  payoutMethodUpsertResultParser,
} from './parsers';
import { UserId } from '../user/types';
import { PAYOUT_METHOD_ORIGIN_TYPES, STRIPE_CONNECT_SESSION_SEPARATOR } from './constants';
import { StripeConnectClientId } from '../stripe/types';
import { ORIGIN_TYPE_STRIPE } from '../origin/constants';
import { PayoutProviderOriginType } from '../payoutProvider/types';

export type PayoutMethodStripeAuthorizeResult = z.infer<typeof payoutMethodStripeAuthorizeResultParser>;

export type StripeConnectAuthSessionString = `${string}${typeof STRIPE_CONNECT_SESSION_SEPARATOR}${UserId}`;

export interface StripeConnectAuthMandatoryParams {
  client_id: StripeConnectClientId;
  state: StripeConnectAuthSessionString;
}

export interface StripeConnectAuthExtraParams {
  'stripe_user[business_type]': 'individual';
  // 'stripe_user[first_name]': user.name ?? 'test user', // TODO find out where user name lives; can't see it in neither auth nor collection
  // 'stripe_user[last_name]': req.user.lastName || undefined,
  'stripe_user[email]': string;
  'stripe_user[business_name]': 'Currents';
  'stripe_user[business_website]': 'alpha.currents.fm';
  redirect_uri: string;
}

export interface StripeConnectAuthParams extends StripeConnectAuthExtraParams, StripeConnectAuthMandatoryParams {}

export type PayoutMethodStripeTokenBody = z.infer<typeof payoutMethodStripeTokenBodyParser>;

export type PayoutMethodUpsertParams = z.infer<typeof payoutMethodUpsertParamsParser>;
export type PayoutMethodUpsertResult = z.infer<typeof payoutMethodUpsertResultParser>;
export type PayoutMethodListQuery = z.infer<typeof payoutMethodListQueryParser>;
export type PayoutMethodListResult = z.infer<typeof payoutMethodListResultParser>;
export type PayoutMethodDeleteParams = z.infer<typeof payoutMethodDeleteParamsParser>;
export type PayoutMethodOriginType = typeof PAYOUT_METHOD_ORIGIN_TYPES[number];
export type PayoutMethodOriginId<PP extends PayoutProviderOriginType = typeof ORIGIN_TYPE_STRIPE> =
  PP extends typeof ORIGIN_TYPE_STRIPE ? StripeConnectClientId : never;
