import {
  StripeConnectClientId,
  StripeConnectTransferId,
  StripeConnectTransferReversalId,
  StripeCustomerId,
  StripeInvoiceId,
  StripePaymentIntentId,
  StripePaymentMethodId,
  StripePriceId,
  StripeProductId,
  StripeSubscriptionId,
} from './types';
import {
  stripeConnectClientIdParser,
  stripeConnectTransferIdParser,
  stripeConnectTransferReversalIdParser,
  stripeCustomerIdParser,
  stripeInvoiceIdParser,
  stripePaymentIntentIdParser,
  stripePaymentMethodIdParser,
  stripePriceIdParser,
  stripeProductIdParser,
  stripeSubscriptionIdParser,
} from './parsers';
import { isFromParser } from '../utils/parsers';

export const isStripeSubscriptionId = (id: string): id is StripeSubscriptionId =>
  isFromParser(stripeSubscriptionIdParser)(id);
export const castStripeSubscriptionId = (id: string): StripeSubscriptionId => {
  if (!isStripeSubscriptionId(id)) throw new Error(`Not a StripeSubscriptionId id ${id}`);
  return id;
};
export const isStripePaymentIntentId = (id: string): id is StripePaymentIntentId =>
  isFromParser(stripePaymentIntentIdParser)(id);
export const castStripePaymentIntentId = (id: string): StripePaymentIntentId => {
  if (!isStripePaymentIntentId(id)) throw new Error(`Not a StripePaymentIntentId id ${id}`);
  return id;
};
export const isStripeInvoiceId = (id: string): id is StripeInvoiceId => isFromParser(stripeInvoiceIdParser)(id);
export const castStripeInvoiceId = (id: string): StripeInvoiceId => {
  if (!isStripeInvoiceId(id)) throw new Error(`Not a StripeInvoiceId id ${id}`);
  return id;
};
export const isStripeCustomerId = (id: string): id is StripeCustomerId => isFromParser(stripeCustomerIdParser)(id);
export const castStripeCustomerId = (id: string): StripeCustomerId => {
  if (!isStripeCustomerId(id)) throw new Error(`Not a StripeCustomerId id ${id}`);
  return id;
};
export const isStripePriceId = (id: string): id is StripePriceId => isFromParser(stripePriceIdParser)(id);
export const isStripeProductId = (id: string): id is StripeProductId => isFromParser(stripeProductIdParser)(id);
export const isStripePaymentMethodId = (id: string): id is StripePaymentMethodId =>
  isFromParser(stripePaymentMethodIdParser)(id);

export const isStripeConnectTransferId = (id: string): id is StripeConnectTransferId =>
  isFromParser(stripeConnectTransferIdParser)(id);
export const castStripeConnectTransferId = (id: string): StripeConnectTransferId => {
  if (!isStripeConnectTransferId(id)) throw new Error(`Not a StripeConnectTransfer id ${id}`);
  return id;
};

export const isStripeConnectTransferReversalId = (id: string): id is StripeConnectTransferReversalId =>
  isFromParser(stripeConnectTransferReversalIdParser)(id);
export const castStripeConnectTransferReversalId = (id: string): StripeConnectTransferReversalId => {
  if (!isStripeConnectTransferReversalId(id)) throw new Error(`Not a StripeConnectTransfer reversal id ${id}`);
  return id;
};

export const isStripeConnectClientId = (id: string): id is StripeConnectClientId =>
  isFromParser(stripeConnectClientIdParser)(id);
export const castStripeConnectClientId = (id: string): StripeConnectClientId => {
  if (!isStripeConnectClientId(id)) throw new Error(`Not a StripeConnectClient id ${id}`);
  return id;
};
